#
#  Copyright (c) 1998, 1999, Digital Creations, Fredericksburg, VA, USA.  
#  All rights reserved.
#  
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#  
#    o Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the disclaimer that follows.
#  
#    o Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions, and the following disclaimer in
#      the documentation and/or other materials provided with the
#      distribution.
#  
#    o Neither the name of Digital Creations nor the names of its
#      contributors may be used to endorse or promote products derived
#      from this software without specific prior written permission.
#  
#  
#  THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS AND CONTRIBUTORS *AS
#  IS* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
#  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
#  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL
#  CREATIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
#  DAMAGE.
#
#  $Id: ociBind.py,v 1.4 2000/05/10 19:14:29 matt Exp $
#
#  If you have questions regarding this software,
#  contact:
# 
#    Digital Creations L.C.  
#    info@digicool.com
# 
#    (540) 371-6909
#
##############################################################################
"""DBI-Compliant Oracle Database Interface
"""

import oci_, dbi, time
from Buffer import Buffer, BufferType
from time import mktime
from string import find
import ocidb, ociCurs
from ociUtil import RowID, error

null=Buffer(1,'h'); null[0]=-1
nonNull=Buffer(1,'h'); nonNull[0]=0

TupleType=type(())
ListType=type([])

def _modified_error(name, v):
    raise error, 'A mutable value was passed to an output'

def _bindFloatScalar(c, name, v, modified):
    arcode=Buffer(1,'H')
    alen=Buffer(1,'H')
    alen[0]=8
    oci_.bind(c, name, v, 8, 4, nonNull, alen, arcode, 0, None)
    return arcode, name, v, alen

def _bindFloatArray(c, name, v, modified):
    l=len(v)
    arcode=Buffer(l,'H')
    alen=Buffer(l,'H')
    ind=Buffer(l,'h')
    buf=Buffer(l,'d')
    for i in range(l):
        alen[i]=8
        o=v[i]
        if o is None: ind[i]=-1
        else:
            ind[i]=0
            buf[i]=o
    oci_.bind(c, name, buf, 8, 4, ind, alen, arcode, 0, None)
    return arcode, name, buf, alen, ind


def _bindBufferScalar(c, name, v, modified, types={
    's': 1, 'b': 3, 'h': 3, 'i': 3, 'f': 4, 'd': 4, 
    }):
    try: ftype=types[v.typecode]
    except:
        raise error, 'Invalid buffer type code for parameter %s' % name
    arcode=Buffer(1,'H')
    alen=Buffer(1,'H')
    ind=Buffer(1,'h')
    alen[0]=v.itemsize
    oci_.bind(c, name, v, alen[0], ftype, ind, alen, arcode, 0, None)
    return arcode, name, v, alen, ind

def _bindIntScalar(c, name, v, modified):
    arcode=Buffer(1,'H')
    alen=Buffer(1,'H')
    alen[0]=4
    oci_.bind(c, name, v, 4, 3, nonNull, alen, arcode, 0, None)
    return arcode, name, v, alen

def _bindIntArray(c, name, v, modified):
    l=len(v)
    arcode=Buffer(l,'H')
    alen=Buffer(l,'H')
    ind=Buffer(l,'h')
    buf=Buffer(l,'i')
    for i in range(l):
        alen[i]=4
        o=v[i]
        if o is None: ind[i]=-1
        else:
            ind[i]=0
            buf[i]=o
    oci_.bind(c, name, buf, 4, 3, ind, alen, arcode, 0, None)
    return arcode, name, buf, alen, ind

def _bindNullScalar(c, name, v, modified):
    arcode=Buffer(1,'H')
    alen=Buffer(1,'H')
    alen[0]=0
    oci_.bind(c, name, v, 0, 1, null, alen, arcode, 0, None)
    return arcode, name, v, alen

def _bindNullArray(c, name, v, modified):
    l=len(v)
    arcode=Buffer(l,'H')
    alen=Buffer(l,'H')
    ind=Buffer(l,'h')
    for i in range(l):
        alen[i]=0
        ind[i]=-1
    oci_.bind(c, name, None, 0, 1, ind, alen, arcode, 0, None)
    return arcode, name, alen, ind

def _bindStringScalar(c, name, v, modified):
    arcode=Buffer(1,'H')
    alen=Buffer(1,'H')
    maxl=len(v)
    if maxl > 2000:
        ot=8 # LONG
    else:
        ot=1 # VARCHAR2
     
    alen[0]=maxl
    oci_.bind(c, name, v, maxl, ot, nonNull, alen, arcode, 0, None)
    return arcode, name, v, alen

def _bindStringArray(c, name, v, raw=0):
    l=len(v)
    arcode=Buffer(l,'H')
    alen=Buffer(l,'H')
    ind=Buffer(l,'h')

    maxl=0
    for o in v:
        if o is not None:
            maxl=max(maxl, len(o))

    if maxl > 32000:
        raise error, 'Values too LONG for array bind of ', name

    if raw:
        # RAW
        if maxl > 255:  ot=24 # LONG RAW
        else:           ot=23 # RAW
    else:
        if maxl > 2000: ot=8 # LONG
        else:           ot=1 # VARCHAR2

    buf=Buffer(maxl*l)
    ii=0
    for i in range(l):
        o=v[i]
        if o is None:
            ind[i]=-1
            alen[i]=0
        else:
            ind[i]=0
            lo=len(o)
            alen[i]=lo
            buf[ii:ii+lo]=o             # MTK patch from Buffer[..] 5/10/00
        ii=ii+maxl
    oci_.bind(c, name, buf, maxl, ot, ind, alen, arcode, 0, None)
    return arcode, name, buf, alen, ind

def _bindInstanceScalar(c, name, v, modified):
    arcode=Buffer(1,'H')
    alen=Buffer(1,'H')
    klass=v.__class__
    if klass is dbi.dbiDate:
        if modified: _modified_error(name, v)
        y,m,d,h,mi,s=(v.year, v.month, v.day,
                      v.hour, v.minute, v.second)
        v=Buffer(7,'B')
        try: v[2]=m
        except:
            # Oops, must have methods
            y=y()
            m=m()
            d=d()
            h=h()
            mi=mi()
            s=s()
            v[2]=m
        
        v[0]=y/100+100
        v[1]=y%100+100
        v[3]=d
        v[4]=h+1
        v[5]=mi+1
        v[6]=s+1
        alen[0]=7
        oci_.bind(c, name, v, 7, 12, nonNull, alen, arcode, 0, None)
    elif klass is dbi.dbiRaw:
        if modified: _modified_error(name, v)
        v=v._v
        maxl=len(v)
        if maxl > 255:
            if maxl > 32000:
                # Very long...
                v=v,Buffer(8192),Buffer(2,'I')
                oci_.bindpw(c,name,v,maxl,24,0,nonNull,arcode,0,None)
                return arcode, name, v
            ot=24 # LONG RAW
        else:
            ot=23 # RAW     
        alen[0]=maxl
        oci_.bind(c, name, v, maxl, ot, nonNull, alen, arcode, 0, None)
    elif klass is RowID:
        if modified: _modified_error(name, v)
        alen[0]=11
        oci_.bind(c, name, v._v, 11, 11, nonNull, alen, arcode, 0, None)
    elif klass is ociCurs.Cursor:
        alen[0]=0
        oci_.bind(c, name, v._c, oci_.CDA_SIZE, oci_.SQLT_CUR,
                  None, None, None, 0, None)
        
    else:
        raise error, 'Unsupported bind class, %s' % klass
    
    return arcode, name, v, alen

def _bindDateArray(c, name, v, modified):
    l=len(v)
    arcode=Buffer(l,'H')
    alen=Buffer(l,'H')
    ind=Buffer(l,'h')
    buf=Buffer(7*l, 'B')
    for i in range(l):
        alen[i]=7
        o=v[i]
        if o is None: ind[i]=-1
        else:
            ind[i]=0
            y,m,d,h,mi,s=o.localtime
            buf[7*i  ]=y/100+100
            buf[7*i+1]=y%100+100
            buf[7*i+2]=m
            buf[7*i+3]=d
            buf[7*i+4]=h+1
            buf[7*i+5]=mi+1
            buf[7*i+6]=s+1

    oci_.bind(c, name, buf, 7, 12, ind, alen, arcode, 0, None)
    return arcode, name, buf, alen, ind

def _bindRawArray(c, name, v, modified):
    return _bindStringArray(c, name, v._v, 1)

def _bindRowIDArray(c, name, v, modified):
    l=len(v)
    arcode=Buffer(l,'H')
    alen=Buffer(l,'H')
    ind=Buffer(l,'h')
    buf=Buffer(11*l)
    for i in range(l):
        alen[i]=11
        o=v[i]
        if o is None: ind[i]=-1
        else:
            ind[i]=0
            buf[i*11:i*11+11]=o._v

    oci_.bind(c, name, buf, 11, 11, ind, alen, arcode, 0, None)
    return arcode, name, buf, alen, ind

def _bindInstanceArray(c, name, v, modified, 
                       binders={
                           dbi.dbiRaw:  _bindRawArray,
                           RowID:       _bindRowIDArray,
                           }):
    for o in v:
        if o is not None: break
    klass=o.__class__
    if klass is dbi.dbiDate: return _bindDateArray(c,name,v)
    if binders.has_key(klass): return binders[klass](c,name,v)
    raise error, 'Unsupported bind class, %s' % klass

def _bindOtherScalar(t, c, name, v, modified):
    raise error, ("Don't know how to bind values of type %s for %s"
                  % (t.__name__, name))

def _bindOtherArray(t, c, name, v, modified):
    raise error, ("Don't know how to bind values of type %s for %s"
                  % (t.__name__, name))

def bindScalar(c, name, v, modified=None):
    pt=type(v)
    m=not not modified
    if _haveScalarBinder((m, pt)):
        return _scalarBinders[m,pt](c, name, v, modified)
    else:
        return _bindOtherScalar(pt, c, name, v, modified)
    

def bindParams(cursor, params):

    # First, make sure they are all the same length, in case of array
    lp=None
    array=0
    for name, p in params:
        if type(p) is TupleType or type(p) is ListType:
            array=1
            if lp is None:
                lp=len(p)
                if not lp:
                    raise error, (
                        'empty parameter '+type(p).__name__)
            elif lp!=len(p):
                raise error, 'parameters have differing lengths'
        else:
            if lp is None: lp=1
            elif lp!=1:
                raise error, 'parameters have differing lengths'


    cruft=[]
    if lp==1:
        # Scalar case
        if array:
            # Convert 1-element arrays to scalars
            sparams=[]
            for name, p in params:
                if type(p) is TupleType or type(p) is ListType: p=p[0]
                sparams.append((name, p))
            params=sparams
        for name, p in params:
            pt=type(p)
            if _haveScalarBinder((0,pt)):
                cruft.append(_scalarBinders[0,pt](cursor._c, name, p, 0))
            else:
                cruft.append(_bindOtherScalar(pt, cursor._c, name, p, 0))

    else:
        # Bind arrays
        i=0
        for name, p in params:
            i=i+1
            pt=None
            for o in p:
                if o is not None:
                    if pt is None:
                        pt=type(o)
                    elif pt is not type(o):
                        raise error, (
                            'parameter %s values have varying type'
                            % type(p).__name__)
            if _haveArrayBinder(pt):
                cruft.append(_arrayBinders[pt](cursor._c, name, p, 0))
            else:
                cruft.append(_bindOtherArray(pt, cursor._c, name, p, 0))

    return lp, cruft


_scalarBinders={
    (0, type(0.0)): _bindFloatScalar,
    (0, type(0)): _bindIntScalar,
    (0, type(None)): _bindNullScalar,
    (1, type(None)): _bindNullScalar,
    (0, type('')): _bindStringScalar,
    (0, type(RowID(''))): _bindInstanceScalar,
    (1, type(RowID(''))): _bindInstanceScalar,
    (0, BufferType): _bindBufferScalar,
    (1, BufferType): _bindBufferScalar,
    }
_haveScalarBinder=_scalarBinders.has_key

_arrayBinders={
    type(0.0): _bindFloatArray,
    type(0): _bindIntArray,
    type(None): _bindNullArray,
    type(''): _bindStringArray,
    type(RowID('')): _bindInstanceArray,
    }
_haveArrayBinder=_arrayBinders.has_key
