/*
**	oci_8.c
**

  Copyright (c) 1998, 1999, 2000 Digital Creations, Fredericksburg, VA, USA.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

    o Redistributions of source code must retain the above copyright
      notice, this list of conditions, and the disclaimer that follows.

    o Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions, and the following disclaimer in
      the documentation and/or other materials provided with the
      distribution.

    o Neither the name of Digital Creations nor the names of its
      contributors may be used to endorse or promote products derived
      from this software without specific prior written permission.


  THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS AND CONTRIBUTORS *AS
  IS* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL
  CREATIONS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
  DAMAGE.

  If you have questions regarding this software,
  contact:

    Digital Creations L.C.
    info@digicool.com

    (540) 371-6909
**
**
** $Id: oci_8.c,v 1.7 2000/06/07 18:27:56 matt Exp $
**
** Maintenance:
**
** 04/05/2000 MTK	Try to work around oracle 8i error during logoff,
**			converted to using Session calls vs Logon()
**
** 06/01/2000 MTK	Fix a OciSvcCtxToLda miss re Zoltan Varga
**
**
** special debugging defines:
**
**	MTKOCIDEBUG	Turn on to enable extra msgs to stderr
**
** NB:
**	Because the rewrite of this follows the extended logon structure,
** 	the overlaying python code could be extended to allocate the
**	various sessions etc. rather than this self-contained implementation
*/

#include "oci.h"

/* static OCIEnv *envhp = NULL; */

static PyObject *
RaiseOCIError(dvoid *svchp, ub4 handle_type)
{
  text errbuf[256];
  sb4 errcode;
  char *c;

  Py_BEGIN_ALLOW_THREADS

  OCIErrorGet (svchp, 1, NULL, &errcode,
	       errbuf, sizeof(errbuf), handle_type);

  Py_END_ALLOW_THREADS

  c = errbuf;
  while (*c && *c != '\n') c++;
  if (*c == '\n') *c = '\0';	/* Nuke that pesky newline */

  PyErr_SetObject(ErrorObject,
		  Py_BuildValue("is", (int)errcode, errbuf));

fprintf(stderr,"%s\n", errbuf);

  return NULL;
}

/* Declarations for objects of type ServerContext */

/*
** changes from prior release:  all pointers are moved here, extra ptrs
** added for long-form declarations.  No more static envhp.  Variable
** names changed to be consistent with oracle examples (ep went to errhp,
** hp went to svchp
*/

typedef struct {
	PyObject_HEAD
	OCIEnv		*envhp;		/* Environment handle (master)	*/
	OCIError	*errhp;		/* Error handle			*/
	OCIServer	*srvhp;		/* Server handle (connection)	*/
	OCISession	*usrhp;		/* User handle (session)	*/
	OCISvcCtx	*svchp;		/* Service Context		*/
} ServerContext;

staticforward PyTypeObject ServerContextType;

typedef struct {
  PyObject_HEAD
  Connection *db;
  OCILobLocator *svchp;
} Lob;
staticforward PyTypeObject LobType;

/* CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC */

static PyObject *
ServerContext_lda(ServerContext *self, PyObject *args)
{
  Connection *db;
  sword status;

  UNLESS(PyArg_ParseTuple(args,"")) return NULL;

  UNLESS(db = PyObject_NEW(Connection, &ConnectionType)) return NULL;
  db->open=OBJECT(self);
  Py_INCREF(self);
  memset(&(db->lda),0,sizeof(db->lda));
  memset(&(db->hda),0,sizeof(db->hda));

  Py_BEGIN_ALLOW_THREADS
  status=OCISvcCtxToLda (self->svchp, self->errhp, &db->lda);
  Py_END_ALLOW_THREADS
  if (status==OCI_ERROR)
    {
      Py_DECREF(db);
      return RaiseOCIError(self->errhp, OCI_HTYPE_ERROR);
    }

  return OBJECT(db);
}

static struct PyMethodDef ServerContext_methods[] = {
  {"lda", (PyCFunction)ServerContext_lda, METH_VARARGS,
   "lda(errorHandle) -- Get a OCI 7 connection"},
  {NULL,		NULL}		/* sentinel */
};

/*
** Logon wrapper to OCI
*/

static PyObject *Logon(PyObject *ignored, PyObject *args) {
	ServerContext *self;
	char *u, *p, *d; 	/* User, password, database */
	int lu, lp, ld;		/* Length of user, password, database */
	sword status;
	sword credentials = OCI_CRED_RDBMS;

	UNLESS(PyArg_ParseTuple(args, "s#s#s#",
		&u, &lu, &p, &lp, &d, &ld)) return NULL;


	UNLESS(self = PyObject_NEW(ServerContext, &ServerContextType))
		return NULL;

	self->envhp = NULL;
	self->errhp = NULL;
	self->svchp = NULL;
	self->srvhp = NULL;
	self->usrhp = NULL;


/*
**  Oracle 8.0.5 initializes like this:
**
**	OCIInitialize(OCI_THREADED, NULL, NULL, NULL, NULL);
**	OCIEnvInit(&self->envhp, OCI_DEFAULT, 0, NULL);
**
**  Oracle 8.1.5 deprecates this, and initializes like this:
**
**	OCIEnvCreate(&self->envhp, OCI_THREADED, NULL, NULL, NULL, NULL, 
**		0, NULL);
**
**  We'll do things the 8.0.5 way since that is still supported under
**  8.1.5
*/

	/* Initialize Oracle */
#ifdef DCORACLE815
	if(OCIEnvCreate(&self->envhp, OCI_THREADED, NULL, NULL, NULL, NULL, 
		0, NULL)==OCI_ERROR) {
		fprintf(stderr,"OCIEnvCreate failed, %s:%d\n",
			__FILE__,__LINE__);
		RaiseOCIError(self->envhp, OCI_HTYPE_ENV);
		Py_DECREF(self);
		return NULL;
	}
#else
	OCIInitialize(OCI_THREADED, NULL, NULL, NULL, NULL);
	if (OCIEnvInit(&self->envhp, OCI_DEFAULT, 0, NULL) != OCI_SUCCESS) {
		fprintf(stderr,"OCIEnvInit failed, %s:%d\n",
			__FILE__,__LINE__);
		RaiseOCIError(self->envhp, OCI_HTYPE_ENV);
		Py_DECREF(self);
		return NULL;
	}

#endif

#ifdef MTKOCIDEBUG
fprintf(stderr,"OCI initialized, envhp=%08x, self @ 0x%08x\n"
	"\tuser %s, pw %s, server %s\n",
	(unsigned long) self->envhp, (unsigned long) self,
	u != NULL ? u : "(null)",
	p != NULL ? p : "(null)",
	d != NULL ? d : "(null)");
#endif

	Py_BEGIN_ALLOW_THREADS
	status=OCIHandleAlloc(self->envhp, (dvoid**)&self->errhp,
		 OCI_HTYPE_ERROR, 0, NULL);
	Py_END_ALLOW_THREADS

	if(status==OCI_ERROR) {
		self->svchp=NULL;
		self->errhp=NULL;
		RaiseOCIError(self->envhp, OCI_HTYPE_ENV);
		Py_DECREF(self); /* Might not be a good thing with self->envhp */
		return NULL;
	}

#if 0
	Py_BEGIN_ALLOW_THREADS
	status=OCILogon(self->envhp, self->errhp, &self->svchp,
		  (text*)u, lu, (text*)p, lp, (text*)d, ld);
	Py_END_ALLOW_THREADS

	if(status==OCI_ERROR) {
		RaiseOCIError(self->errhp, OCI_HTYPE_ERROR);
		self->svchp=NULL;
		Py_DECREF(self);
		return NULL;
	}
#else
/* This looks kinda yucky but is ok temporary glue */
#define printError(x) RaiseOCIError(x, OCI_HTYPE_ENV); Py_DECREF(self); return NULL;
#define printError1(x,y) RaiseOCIError(x, y); Py_DECREF(self); return NULL;

        if ((status = OCIHandleAlloc(self->envhp, (void **) &self->srvhp,
                OCI_HTYPE_SERVER, 0, NULL)) != OCI_SUCCESS) {
                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
                printError(self->envhp);
        }

	
	if (ld == 0) {
		d = NULL;		/* Safety measure	*/
	}

	Py_BEGIN_ALLOW_THREADS
        status = OCIServerAttach(self->srvhp, self->errhp, d, ld, OCI_DEFAULT);
	Py_END_ALLOW_THREADS

        if (status != OCI_SUCCESS) {
                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
                printError1(self->errhp,OCI_HTYPE_ERROR);
        }

        if ((status = OCIHandleAlloc(self->envhp, (void **) &self->svchp,
                OCI_HTYPE_SVCCTX, 0, NULL)) != OCI_SUCCESS) {
                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
                printError(self->envhp);
        }

        if ((status = OCIAttrSet(self->svchp, OCI_HTYPE_SVCCTX,
                (dvoid *)self->srvhp, (ub4) 0,
                OCI_ATTR_SERVER, self->errhp)) != OCI_SUCCESS) {
                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
                printError(self->envhp);
        }

        if ((status = OCIHandleAlloc(self->envhp, (void **) &self->usrhp,
                OCI_HTYPE_SESSION, 0, NULL)) != OCI_SUCCESS) {
                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
                printError(self->envhp);
        }

	if (u[0] || p[0]) {	/* Were RDBMS credentials provided? */

		if ((status = OCIAttrSet(self->usrhp, OCI_HTYPE_SESSION,
			(dvoid *)u, (ub4) lu,
			OCI_ATTR_USERNAME, self->errhp)) != OCI_SUCCESS) {
			fprintf(stderr,"OCI_ERROR at %s:%d\n",
				__FILE__,__LINE__);
			printError(self->envhp);
		}

		if ((status = OCIAttrSet(self->usrhp, OCI_HTYPE_SESSION,
			(dvoid *)p, (ub4) lp,
			OCI_ATTR_PASSWORD, self->errhp)) != OCI_SUCCESS) {
			fprintf(stderr,"OCI_ERROR at %s:%d\n",
				__FILE__,__LINE__);
			printError(self->envhp);
		}
	} else {
		credentials = OCI_CRED_EXT;	/* Use external credentials */
	}

	Py_BEGIN_ALLOW_THREADS
        status = OCISessionBegin(self->svchp, self->errhp, self->usrhp,
                credentials, OCI_DEFAULT);
	Py_END_ALLOW_THREADS

        if (status  != OCI_SUCCESS) {
                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
		RaiseOCIError(self->errhp, OCI_HTYPE_ERROR);
		/*Py_DECREF(self);*/
		return NULL;
        }

        if ((status = OCIAttrSet(self->svchp, OCI_HTYPE_SVCCTX,
                self->usrhp, 0, OCI_ATTR_SESSION,
                self->errhp)) != OCI_SUCCESS) {

                fprintf(stderr,"OCI_ERROR at %s:%d\n", __FILE__,__LINE__);
                printError(self->envhp);
        }


#endif

#ifdef MTKOCIDEBUG
fprintf(stderr,"Logon has happened, returning %08x, svchp=%08x, errhp=%08x\n",
	(unsigned long) self,
	(unsigned long) self->svchp,
	(unsigned long) self->errhp);
#endif
 	
	return OBJECT(self);
}

static void
ServerContext_dealloc(ServerContext *self)
{
	int errors = 0;
	sword status;
	sb4 errcode;
	char errbuf[256];

#ifdef MTKOCIDEBUG
fprintf(stderr,"ServerContext_dealloc is happening, self at %08x, "
	"svchp=%08x, errhp=%08x\n",
	(unsigned long) self,
	(unsigned long) self->svchp,
	(unsigned long) self->errhp);
#endif
 
#if 0
	Py_BEGIN_ALLOW_THREADS
	status = OCILogoff(self->svchp, self->errhp);
	if (status == OCI_ERROR) {
		OCIErrorGet(self->envhp, 1, NULL, &errcode, errbuf,
			sizeof(errbuf), OCI_HTYPE_ENV);
		fprintf(stderr,"Error in OCILogoff, code %d: %s\n",
				errcode, errbuf);
		errors = 1;
	}
		
	if (self->errhp) {
		status = OCIHandleFree(self->errhp, OCI_HTYPE_ERROR);
		if (status == OCI_ERROR) {
			OCIErrorGet(self->envhp, 1, NULL, &errcode, errbuf,
				sizeof(errbuf), OCI_HTYPE_ENV);
			fprintf(stderr,"Error in OCIHandleFree, code %d: %s\n",
				errcode, errbuf);
			errors = 1;
		}
	}
	Py_END_ALLOW_THREADS

	if (OCIHandleFree(self->envhp, OCI_HTYPE_ENV) != OCI_SUCCESS) {
		fprintf(stderr,"OCIHandleFree failed to release "
			"environment handle " "at 0x%08x\n", self->envhp);
	}

#else

	if (self->svchp) {
		Py_BEGIN_ALLOW_THREADS
		status = OCISessionEnd(self->svchp, self->errhp, self->usrhp,
			OCI_DEFAULT);
		Py_END_ALLOW_THREADS

		if (status != OCI_SUCCESS) {
			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
			OCIErrorGet(self->envhp, 1, NULL, &errcode, errbuf,
				sizeof(errbuf), OCI_HTYPE_ENV);
			fprintf(stderr,"Error in OCISessionEnd, code %d: %s\n",
				errcode, errbuf);
		}
	}

	if (self->srvhp) {
		Py_BEGIN_ALLOW_THREADS
		status = OCIServerDetach(self->srvhp, self->errhp, OCI_DEFAULT);
		Py_END_ALLOW_THREADS
		if (status != OCI_SUCCESS) {
			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
			OCIErrorGet(self->envhp, 1, NULL, &errcode, errbuf,
				sizeof(errbuf), OCI_HTYPE_ENV);
			fprintf(stderr,"Error in OCIServerDetach, "
				"code %d: %s\n", errcode, errbuf);
		}
	}

	if (self->svchp) 
	if ((status = OCIHandleFree(self->svchp, OCI_HTYPE_SVCCTX)) !=
		OCI_SUCCESS) {

			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
	}

	if (self->errhp)
        if ((status = OCIHandleFree(self->errhp, OCI_HTYPE_ERROR)) !=
                OCI_SUCCESS) {

			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
        }

	if (self->srvhp)
        if ((status = OCIHandleFree(self->srvhp, OCI_HTYPE_SERVER)) !=
                OCI_SUCCESS) {

			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
        }

	if (self->usrhp)
        if ((status = OCIHandleFree(self->usrhp, OCI_HTYPE_SESSION)) !=
                OCI_SUCCESS) {

			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
        }

	if (self->envhp)
        if ((status = OCIHandleFree(self->envhp, OCI_HTYPE_ENV)) !=
                OCI_SUCCESS) {

			fprintf(stderr,"OCI_ERROR %d at %s:%d\n", status,
				__FILE__,__LINE__);
        }


#endif

  PyMem_DEL(self);
}

static PyObject *
ServerContext_getattr(ServerContext *self, char *cname)
{
  return Py_FindMethod(ServerContext_methods, (PyObject *)self, cname);
}

static char ServerContexttype__doc__[] = 
"An Oracle 8 database connection"
;

static PyTypeObject ServerContextType = {
  PyObject_HEAD_INIT(NULL)
  0,				/*ob_size*/
  "ServerContext",			/*tp_name*/
  sizeof(ServerContext),		/*tp_basicsize*/
  0,				/*tp_itemsize*/
  /* methods */
  (destructor)ServerContext_dealloc,	/*tp_dealloc*/
  (printfunc)0,	/*tp_print*/
  (getattrfunc)ServerContext_getattr,		/*obsolete tp_getattr*/
  (setattrfunc)0,		/*obsolete tp_setattr*/
  (cmpfunc)0,	/*tp_compare*/
  (reprfunc)0,		/*tp_repr*/
  0,		/*tp_as_number*/
  0,		/*tp_as_sequence*/
  0,		/*tp_as_mapping*/
  (hashfunc)0,		/*tp_hash*/
  (ternaryfunc)0,	/*tp_call*/
  (reprfunc)0,		/*tp_str*/
  (getattrofunc)0,	/*tp_getattro*/
  (setattrofunc)0,	/*tp_setattro*/
  
  /* Space for future expansion */
  0L,0L,
  ServerContexttype__doc__ /* Documentation string */
};

/* LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL */

static PyObject *
Lob_define(Lob *self, PyObject *args)
{
  Cursor *c;
  Buffer *indp, *alen, *arcode;
  int pos, ftype;
  ServerContext *sc;
  sword status;

  UNLESS(PyArg_ParseTuple(args,"O!iiO!O!O!",
			  &CursorType, &c, &pos, &ftype,
			  BufferType, &indp, BufferType, &alen, BufferType, &arcode
			  )) return NULL;

  Py_BEGIN_ALLOW_THREADS
  status=odefinps(&c->cda, 1, pos, (ub1*)&self->svchp,
		  sizeof(OCILobLocator *), ftype, 0, 
		  indp->val.h, NULL, 0, 0,
		  alen->val.H, arcode->val.H,
		  sizeof(OCILobLocator *), 2, 2, 2);
  Py_END_ALLOW_THREADS

  return PyInt_FromLong(status);
}

static PyObject *
Lob_write(Lob *self, PyObject *args)
{
  ub4 amtp;
  char *s;
  int l, csid=0, csfrm=SQLCS_IMPLICIT, offset=1;
  sword status;
  PyObject *r;
  ServerContext *sc;

  UNLESS(PyArg_ParseTuple(args,"s#|iii", &s, &l, &offset, &csid, &csfrm)) return NULL;
  amtp=l;
	
  if (! self->db->open || self->db->open->ob_type != &ServerContextType)
    {
      PyErr_SetString(ErrorObject,
		      "Database connection is clodes or not based on a\n"
		      "OCI 8 server context");
      return NULL;
    }
  sc=(ServerContext*)(self->db->open);

  Py_BEGIN_ALLOW_THREADS
  status=OCILdaToSvcCtx(&sc->svchp, sc->errhp, &self->db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  Py_BEGIN_ALLOW_THREADS
  status=OCILobWrite(sc->svchp, sc->errhp, self->svchp, &amtp, offset, s, l,
		     OCI_ONE_PIECE, NULL, NULL, csid, csfrm);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) {
    RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);
    r=NULL;
  }
  else r=PyInt_FromLong(status);

  Py_BEGIN_ALLOW_THREADS
  status=OCISvcCtxToLda(sc->svchp, sc->errhp, &self->db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  return r;
}
    
static PyObject *
Lob_read(Lob *self, PyObject *args)
{
  ub4 amtp, ll;
  int l=-1, csid=0, csfrm=SQLCS_IMPLICIT, offset=1;
  sword status;
  PyObject *r=NULL;
  char *s;
  ServerContext *sc;
  
  UNLESS(PyArg_ParseTuple(args,"|iiii", &l, &offset, &csid, &csfrm)) return NULL;
	
  if (! self->db->open || self->db->open->ob_type != &ServerContextType)
    {
      PyErr_SetString(ErrorObject,
		      "Database connection is clodes or not based on a\n"
		      "OCI 8 server context");
      return NULL;
    }
  sc=(ServerContext*)(self->db->open);

  Py_BEGIN_ALLOW_THREADS
  status=OCILdaToSvcCtx(&sc->svchp, sc->errhp, &self->db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  Py_BEGIN_ALLOW_THREADS
  status=OCILobGetLength(sc->svchp, sc->errhp, self->svchp, &ll);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR)
    {
      RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);
      goto finally;
    }
  ll+=offset-1;
  if (l <= 0 || l > ll) l=ll;

  if (l < 1) { 
  	r = PyString_FromString("");
	goto finally;	/* Fix re: Zoltan Varga */
  }

  amtp=l;
  r=PyString_FromStringAndSize(NULL, l);
  s=PyString_AsString(r);
  Py_BEGIN_ALLOW_THREADS
  status=OCILobRead(sc->svchp, sc->errhp, self->svchp, &amtp, offset, s, l,
		    NULL, NULL, csid, csfrm);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) {
    RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);
    Py_DECREF(r);
    r=NULL;
  }

finally:
  Py_BEGIN_ALLOW_THREADS
  status=OCISvcCtxToLda(sc->svchp, sc->errhp, &self->db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  return r;
}
    
static PyObject *
Lob_length(Lob *self, PyObject *args)
{
  ub4 ll;
  sword status;
  PyObject *r=NULL;
  char *s;
  ServerContext *sc;
  
  UNLESS(PyArg_ParseTuple(args,"")) return NULL;
	
  if (! self->db->open || self->db->open->ob_type != &ServerContextType)
    {
      PyErr_SetString(ErrorObject,
		      "Database connection is clodes or not based on a\n"
		      "OCI 8 server context");
      return NULL;
    }
  sc=(ServerContext*)(self->db->open);

  Py_BEGIN_ALLOW_THREADS
  status=OCILdaToSvcCtx(&sc->svchp, sc->errhp, &self->db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  Py_BEGIN_ALLOW_THREADS
  status=OCILobGetLength(sc->svchp, sc->errhp, self->svchp, &ll);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR)
    {
      RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);
      goto finally;
    }
  r=PyInt_FromLong(ll);

finally:
  Py_BEGIN_ALLOW_THREADS
  status=OCISvcCtxToLda(sc->svchp, sc->errhp, &self->db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  return r;
}

static struct PyMethodDef Lob_methods[] = {
  {"length", (PyCFunction)Lob_length, METH_VARARGS,
   "length(errorHandle) -- Get the LOB length"},
  {"read", (PyCFunction)Lob_read, METH_VARARGS,
   "read(errorHandle, [n, offset, csid, csfrm) -- "
   "read from the LOB\n\n"
   "read n (or all) bytes at some offset (from 1) using\n"
   "a character set id(0) and character set form (SQLCS_IMPLICIT)\n"
  },
  {"write", (PyCFunction)Lob_write, METH_VARARGS,
   "read(errorHandle, s, [offset, csid, csfrm) -- "
   "write a string, s, to the LOB\n\n"
   "read n (or all) bytes at some offset (from 1) using\n"
   "a character set id(0) and character set form (SQLCS_IMPLICIT)\n"
  },
  {"define", (PyCFunction)Lob_define, METH_VARARGS,
   "define(cursor, position, type, ind, alen, arcode) -- "
   "define an output binding"
  },
  {NULL,		NULL}		/* sentinel */
};

static void
Lob_dealloc(Lob *self)
{
  if(self->db) Py_DECREF(self->db);		/* MTK */
  if(self->svchp)
    {
      Py_BEGIN_ALLOW_THREADS
      OCIDescriptorFree(self->svchp, OCI_DTYPE_LOB);
      Py_END_ALLOW_THREADS
    }
  PyMem_DEL(self);
}

static PyObject *
Lob_getattr(Lob *self, char *cname)
{
  return Py_FindMethod(Lob_methods, (PyObject *)self, cname);
}

static char Lobtype__doc__[] = 
"Oracle Lob";

static PyTypeObject LobType = {
  PyObject_HEAD_INIT(NULL)
  0,				/*ob_size*/
  "Lob",			/*tp_name*/
  sizeof(Lob),		/*tp_basicsize*/
  0,				/*tp_itemsize*/
  /* methods */
  (destructor)Lob_dealloc,	/*tp_dealloc*/
  (printfunc)0,	/*tp_print*/
  (getattrfunc)Lob_getattr,		/*obsolete tp_getattr*/
  (setattrfunc)0,		/*obsolete tp_setattr*/
  (cmpfunc)0,	/*tp_compare*/
  (reprfunc)0,		/*tp_repr*/
  0,		/*tp_as_number*/
  0,		/*tp_as_sequence*/
  0,		/*tp_as_mapping*/
  (hashfunc)0,		/*tp_hash*/
  (ternaryfunc)0,	/*tp_call*/
  (reprfunc)0,		/*tp_str*/
  (getattrofunc)0,	/*tp_getattro*/
  (setattrofunc)0,	/*tp_setattro*/
  
  /* Space for future expansion */
  0L,0L,
  Lobtype__doc__ /* Documentation string */
};

/* MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM */

static PyObject *
newLob(PyObject *self, PyObject *args)
{
  Connection *db;
  Lob *lob;
  ServerContext *sc;
  sword status;

  UNLESS(PyArg_ParseTuple(args,"O!", &ConnectionType, &db)) return NULL;
	
  if (! db->open || db->open->ob_type != &ServerContextType)
    {
      PyErr_SetString(ErrorObject,
		      "Database connection is clodes or not based on a\n"
		      "OCI 8 server context");
      return NULL;
    }
  sc=(ServerContext*)(db->open);

  Py_BEGIN_ALLOW_THREADS
  status=OCILdaToSvcCtx(&sc->svchp, sc->errhp, &db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR) return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);

  UNLESS(lob = PyObject_NEW(Lob, &LobType)) return NULL;
  lob->db=db;
  Py_INCREF(db);
  Py_BEGIN_ALLOW_THREADS
  status=OCIDescriptorAlloc(sc->envhp, (dvoid**)&lob->svchp, OCI_DTYPE_LOB, 0, NULL);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR)
    {
      lob->svchp=NULL;
      Py_DECREF(lob);
      return RaiseOCIError(sc->envhp, OCI_HTYPE_ENV);
    }

  Py_BEGIN_ALLOW_THREADS
  status=OCISvcCtxToLda(sc->svchp, sc->errhp, &db->lda);
  Py_END_ALLOW_THREADS
  if(status==OCI_ERROR)
    {
      Py_DECREF(lob);
      return RaiseOCIError(sc->errhp, OCI_HTYPE_ERROR);
    }

  return OBJECT(lob);
}

static struct PyMethodDef Module_Level__methods[] = {
  {"lob", (PyCFunction)newLob, METH_VARARGS,
   "allocate a Lob descriptor"},
  {"Logon", (PyCFunction)Logon, METH_VARARGS,
   "Logon to the database"},
  {NULL, (PyCFunction)NULL, 0, NULL}		/* sentinel */
};

/*
**	Initialization
**
*/

void
initoci_8()
{
  PyObject *m, *d;
  char *rev="$Revision: 1.7 $";


#if 0
  if (envhp != NULL) {
	fprintf(stderr,"initoci_8 ignoring second invocation\n");
	return;
   }

  /* Initialize Oracle  -- */
  OCIInitialize(OCI_THREADED, NULL, NULL, NULL, NULL);
  if(OCIEnvInit(&envhp, OCI_DEFAULT, NULL, NULL)==OCI_ERROR)
    {
      RaiseOCIError(envhp);
      return;
    }
fprintf(stderr,"OCI initialized, envhp=%08x\n", (unsigned long) envhp);
#endif


  /* Create the module and add the functions */
  m = Py_InitModule4("oci_8", Module_Level__methods,
		     "hack module to get access to OCI 8 LOBs",
		     (PyObject*)NULL,PYTHON_API_VERSION);

  /* Add some symbolic constants to the module */
  d = PyModule_GetDict(m);


  ServerContextType.ob_type=&PyType_Type;
  PyDict_SetItemString(d,"ServerContextType", OBJECT(&ServerContextType));

  LobType.ob_type=&PyType_Type;
  PyDict_SetItemString(d,"LobType", OBJECT(&LobType));

  PyDict_SetItemString(d, "error", ErrorObject);

  PyDict_SetItemString(d, "__version__",
		       PyString_FromStringAndSize(rev+11,strlen(rev+11)-2));
  
	
  /* Check for errors */
  if (PyErr_Occurred())
    Py_FatalError("can't initialize module oci_8");
}
